module CURIC::SectionPlane
	module Display
    def self.show_section_planes
      m = Sketchup.active_model

      data_by_distance = self.get_section_planes()
      return unless data_by_distance && !data_by_distance.empty?

			if m.rendering_options["DisplaySectionPlanes"]
				m.rendering_options["DisplaySectionPlanes"] = false
			else
				m.rendering_options["DisplaySectionPlanes"] = true

				m.start_operation("show", true)
				data_by_distance.keys.each do |section_plane| 
					section_plane.hidden = false
					section_plane.layer.visible = true 
				end
				m.commit_operation
      end
    end

    def self.get_section_planes
      m = Sketchup.active_model
      es = m.entities
      view = m.active_view
      cam = view.camera

      section_planes = es.grep(Sketchup::SectionPlane)
      return UI.messagebox("No SectionPlane") if section_planes.empty?

      current_section_plane = self.get_current()
      return unless current_section_plane
    
      # get all section_plane samedirection
      section_planes_samedirection = section_planes.find_all { |sp| 
        section_plane_vec(sp).samedirection?( section_plane_vec(current_section_plane) ) 
      }

      # remove same plane
      point = m.bounds.min
      data_by_distance = section_planes_samedirection.each_with_object({}){|sp, h|
        plane = sp.get_plane
        d = point.distance_to_plane(plane)
        h[sp] = d.round(1)
      }
      data_by_distance = data_by_distance.sort_by { |key, value| value }.to_h
      data_by_distance = data_by_distance.to_a.uniq{|v| v.last }.to_h
      data_by_distance
    end

		def self.get_current
			cam = Sketchup.active_model.active_view.camera
			section_planes = Sketchup.active_model.entities.grep(Sketchup::SectionPlane)
      current_section_plane = Sketchup.active_model.entities.active_section_plane
      if !current_section_plane
        # get section_plane by current view
        cam.direction
        data = section_planes.each_with_object({}){|section_plane, h|
          plane = section_plane.get_plane
          vec_section_plane = Geom::Vector3d.new(plane[0], plane[1], plane[2])
          angle = cam.direction.angle_between(vec_section_plane)
          h[section_plane] = angle.radians
        }
        data = data.sort_by { |key, value| value }.to_h
        current_section_plane = data.key(data.values.min)
      end

      current_section_plane
    end

    def self.cut_next
      current_section_plane = self.get_current()
      return unless current_section_plane

      m = Sketchup.active_model

      # tìm 2 section_plane gần current_section_plane nhất
      plane = current_section_plane.get_plane
      vec_section_plane = Geom::Vector3d.new(plane[0], plane[1], plane[2])
      point = m.bounds.min.project_to_plane(plane)
      

      nearest_section_plane = get_nearest_section_plane(current_section_plane)
			return unless nearest_section_plane

      section_plane = nil
      if nearest_section_plane.first
        pl = nearest_section_plane.first.get_plane
        point_on_section_plane = point.project_to_plane(pl)
        vec = point_on_section_plane.vector_to(point)
        if vec && vec.valid? && vec.samedirection?(vec_section_plane)
          section_plane = nearest_section_plane.first
        end
      end

      if section_plane.nil? && nearest_section_plane.last
        pl = nearest_section_plane.last.get_plane
        point_on_section_plane = point.project_to_plane(pl)
        vec = point_on_section_plane.vector_to(point)
        if vec && vec.valid? && vec.samedirection?(vec_section_plane)
          section_plane = nearest_section_plane.last
        end
      end

			if section_plane
				m.start_operation("cut_next", true)
        m.rendering_options["DisplaySectionCuts"] = true
				m.entities.active_section_plane=section_plane
				m.commit_operation
      end

      section_plane
    end

    def self.cut_previous
      current_section_plane = self.get_current()
      return unless current_section_plane

      m = Sketchup.active_model

      # tìm 2 section_plane gần current_section_plane nhất
      plane = current_section_plane.get_plane
      vec_section_plane = Geom::Vector3d.new(plane[0], plane[1], plane[2])
      point = m.bounds.min.project_to_plane(plane)
      
      nearest_section_plane = get_nearest_section_plane(current_section_plane)
			return unless nearest_section_plane

      section_plane = nil
      if nearest_section_plane.first
        pl = nearest_section_plane.first.get_plane
        point_on_section_plane = point.project_to_plane(pl)
        vec = point_on_section_plane.vector_to(point)
        if vec && vec.valid? && !vec.samedirection?(vec_section_plane)
          section_plane = nearest_section_plane.first
          d1 = point.distance_to_plane(pl)
        end
      end

      if section_plane.nil? && nearest_section_plane.last
        pl = nearest_section_plane.last.get_plane
        point_on_section_plane = point.project_to_plane(pl)
        vec = point_on_section_plane.vector_to(point)
        if vec && vec.valid? && !vec.samedirection?(vec_section_plane)
          section_plane = nearest_section_plane.last
        end
      end

			if section_plane
				m.start_operation("cut_previous", true)
        m.rendering_options["DisplaySectionCuts"] = true
				m.entities.active_section_plane=section_plane
				m.commit_operation
      end
      
      section_plane
    end

    def self.get_nearest_section_plane(current_section_plane)
      data_by_distance = self.get_section_planes()
      if data_by_distance.length < 2
        UI.messagebox('The number of section planes in the current direction is less than 2!')
      end
      return unless data_by_distance && data_by_distance.length > 1

      index = data_by_distance.keys.index(current_section_plane)
      if index == 0
        return [data_by_distance.keys[1]]
      elsif index == data_by_distance.keys.length-1
        return [data_by_distance.keys[-2]]
      else
        section_plane1 = data_by_distance.keys[index-1]
        section_plane2 = data_by_distance.keys[index+1]
        return [section_plane1, section_plane2]
      end
    end

    def self.section_plane_vec(section_plane)
      plane = section_plane.get_plane
      Geom::Vector3d.new(plane[0], plane[1], plane[2])
    end
	end
end